// WordRecord.c
#include "WordRecord.h"
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <inttypes.h>

void WordRecord_Init(WordRecord* const pWR, const char* const pWord, uint32_t Frequency) {

   assert( pWR != NULL && pWord != NULL);

   pWR->Word = malloc( (strlen(pWord) + 1) * sizeof(char) );
   assert( pWR->Word != NULL );

   strcpy(pWR->Word, pWord);
   pWR->Freq = Frequency;
}

char* WordRecord_Word(const WordRecord* const pWR) {

   assert( pWR != NULL );

   char* copy = malloc( (strlen(pWR->Word) + 1) * sizeof(char) );
   assert( pWR->Word != NULL );

   strcpy(copy, pWR->Word);
   return copy;
}

uint32_t WordRecord_Frequency(const WordRecord* const pWR) {

   assert( pWR != NULL );
   return pWR-> Freq;
}

bool WordRecord_Increment(WordRecord* const pWR) {

   assert( pWR != NULL );

   if ( pWR->Freq == UINT32_MAX ) return false;
   pWR->Freq++;
   return true;
}

bool WordRecord_Decrement(WordRecord* const pWR) {

   assert( pWR != NULL );

   if ( pWR->Freq == 0 ) return false;
   pWR->Freq--;
   return true;
}

bool WordRecord_equals(const WordRecord* const pLeft, const WordRecord* const pRight) {

   return (pLeft->Freq == pRight->Freq &&
           strcmp(pLeft->Word, pRight->Word) == 0 );
}

void WordRecord_Clear(WordRecord* const pWR) {

   assert( pWR != NULL );

   free( pWR->Word );
   pWR->Word = NULL;
   pWR->Freq = 0;
}

void WordRecord_Print(const WordRecord* const pWR, FILE *fp) {

   assert( pWR != NULL && fp != NULL );

   fprintf(fp, "%5"PRIu32": %s", pWR->Freq, pWR->Word);
}

